//---------------------------------------------------------------------------

#include <vcl.h>
#include <FlexControls.hpp>
#pragma hdrstop

#include "Main.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "FlexBase"
#pragma resource "*.dfm"
TfmMain *fmMain;
//---------------------------------------------------------------------------
__fastcall TfmMain::TfmMain(TComponent* Owner)
        : TForm(Owner)
{
  LoadFlexCursors();
  fpMain->InDesign = true;
  FColors = new TList();
  RefreshColors();
  CheckTools();
  DocChange();
  fpMain->Modified = false;
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::dgColorsDrawCell(TObject *Sender, int ACol,
      int ARow, TRect &Rect, TGridDrawState State)
{
  int Index = ACol * dgColors->RowCount + ARow;
  if (Index < FColors->Count)
  {
    dgColors->Canvas->Pen->Color = clBlack;
    dgColors->Canvas->Pen->Style = psSolid;
    dgColors->Canvas->Brush->Style = bsClear;
    dgColors->Canvas->Rectangle(Rect);
    InflateRect(&Rect, -1, -1);
    TColor Color = (TColor)FColors->Items[Index];
    switch (Color) {
      case clNone:
        {
          dgColors->Canvas->Brush->Color = clWhite;
          dgColors->Canvas->Brush->Style = bsSolid;
          dgColors->Canvas->FillRect(Rect);
          dgColors->Canvas->MoveTo(Rect.Left, Rect.Top);
          dgColors->Canvas->LineTo(Rect.Right, Rect.Bottom);
          dgColors->Canvas->MoveTo(Rect.Right, Rect.Top-1);
          dgColors->Canvas->LineTo(Rect.Left-1, Rect.Bottom);
          break;
        }
     case clDefault:
        {
          dgColors->Canvas->Brush->Color = clWhite;
          dgColors->Canvas->Brush->Style = bsSolid;
          dgColors->Canvas->FillRect(Rect);
          InflateRect(&Rect, -4, 0);
          Rect.Bottom -= 2;
          Rect.Top = Rect.Bottom - 2;
          dgColors->Canvas->Brush->Color = clBlack;
          dgColors->Canvas->FillRect(Rect);
        }
     default:
        {
          dgColors->Canvas->Brush->Color = (TColor)ColorToRGB(Color);
          dgColors->Canvas->Brush->Style = bsSolid;
          dgColors->Canvas->FillRect(Rect);
        }
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::RefreshColors()
{
  int i;
  int ARows, ACols;
  FColors->Clear();
  // Add transparent color
  FColors->Add((void *)clNone);
  // Add default colors
  for (i = 0; i < DefaultPaletteCount; i++)
    FColors->Add((void *)ColorToRGB((TColor)DefaultPalette[i]));
  // Add custom colors
  /* for i:=0 to CustomColors.Count-1 do begin
   s := CustomColors[i];
   j := Pos('=', s);
   if j = 0 then continue;
   Color := StrToIntDef('$' + copy(s, j+1, Length(s)-j), -1);
   if Color = -1 then continue;
   FColors.Add(pointer(ColorToRGB(Color)));
  end; */
  // Add default (color edit) color
  FColors->Add((void *)clDefault);
  // Rebuild color grid
  dgColors->RowCount = FColors->Count;
  i = (dgColors->DefaultRowHeight + dgColors->GridLineWidth);
  if (i > 0)
    ARows = dgColors->ClientHeight / i;
  else
    ARows = 0;
  if (ARows > 0)
    ACols = (FColors->Count / ARows) + ((FColors->Count % ARows) ? 1 : 0);
  else
    ACols = 0;
  dgColors->RowCount = ARows;
  dgColors->ColCount = ACols;
  dgColors->Invalidate();
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::panColorsResize(TObject *Sender)
{
  RefreshColors();
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::sptMainMoved(TObject *Sender)
{
  RefreshColors();
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::dgColorsMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
  int Index, i;
  int ACol, ARow;
  TColor Color;
  TFlexControl *Control;
  TBrushProp *Brush;
  TPenProp *Pen;
  TPassControlRec PassRec;

  dgColors->MouseToCell(X, Y, ACol, ARow);
  Index = ACol * dgColors->RowCount + ARow;
  if ((Index >= 0) && (Index < FColors->Count))
    Color = (TColor)FColors->Items[Index];
  else
    return;

  if (fpMain->SelectedCount == 0) return;
  if (Color == clDefault)
  {
    // Define initial color
    Color = clBlack;
    for (i = 0; i < fpMain->SelectedCount; i++)
    {
      Control = fpMain->Selected[i];
      FirstControl(Control, PassRec);
      while (Control)
      {
        Brush = (TBrushProp *) Control->Props->Props["Brush"];
        Pen = (TPenProp *) Control->Props->Props["Pen"];
        if ((Button == mbLeft) && (Brush) && (Brush->Method == bmHatch))
        {
          // Get brush color
          Color = Brush->Color;
          if (Brush->Style != bsClear) break;
        }
        else
        if ((Button == mbRight) && (Pen))
        {
          // Get pen color
          Color = Pen->Color;
          if (Pen->Style != psClear) break;
        }
        // Next sub-control
        Control = NextControl(PassRec, false);
      }
      ClosePassRec(PassRec);
    }
    cd_Palette->Color = Color;
    // Update custom colors
    // cd_Palette.CustomColors.Assign(CustomColors);
    if (cd_Palette->Execute())
    {
     // CustomColors.Assign(cd_Palette.CustomColors);
     Color = cd_Palette->Color;
    }
    else
      return;
  }
  // Set color for selected controls
  for (i = 0; i < fpMain->SelectedCount; i++)
  {
    Control = fpMain->Selected[i];
    FirstControl(Control, PassRec);
    while (Control)
    {
      Brush = (TBrushProp *) Control->Props->Props["Brush"];
      Pen = (TPenProp *) Control->Props->Props["Pen"];
      if ((Button == mbLeft) && (Brush) && (Brush->Method == bmHatch))
      {
        // Fill
        Brush->Method = bmHatch;
        if (Color == clNone)
          Brush->Style = bsClear;
        else
        {
          Brush->Style = bsSolid;
          Brush->Color = (TColor)ColorToRGB(Color);
        }
      }
      else
      if ((Button == mbRight) && (Pen))
      {
        // Outline
        if (Color == clNone)
          Pen->Style = psClear;
        else
        {
          Pen->Style = psSolid;
          Pen->Color = (TColor)ColorToRGB(Color);
        }
      }
      // Next sub-control
      Control = NextControl(PassRec, false);
    }
    ClosePassRec(PassRec);
  }
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::tbtToolClick(TObject *Sender)
{
  //
  if (Sender) ((TToolButton *)Sender)->Down = true;
  if (Sender == tbtZoom)
  {
    fpMain->EditPointControl = NULL;
    fpMain->CppCreatingControlClass = NULL;
    fpMain->ToolMode = ftmZoom;
  }
  else
  if (Sender == tbtPan)
  {
    //fpMain->EditPointControl = NULL;
    fpMain->CppCreatingControlClass = NULL;
    fpMain->ToolMode = ftmPan;
  }
  else
  if (Sender == tbtShape)
  {
    fpMain->CppCreatingControlClass = NULL;
    //if SelectedCount = 1 then EditPointControl = Selected[0];
    fpMain->ToolMode = ftmPointEdit;
  }
  else
  {
    if (Sender == tbtArrow)
    {
      fpMain->CppCreatingControlClass = NULL;
      fpMain->EditPointControl = NULL;
    }
    else
    if (Sender == tbtLine)
      fpMain->CppCreatingControlClass = (unsigned int) __classid(TFlexCurve);
    else
    if (Sender == tbtBox)
      fpMain->CppCreatingControlClass = (unsigned int) __classid(TFlexBox);
    else
    if (Sender == tbtEllipse)
      fpMain->CppCreatingControlClass = (unsigned int) __classid(TFlexEllipse);
    else
    if (Sender == tbtText)
      fpMain->CppCreatingControlClass = (unsigned int) __classid(TFlexText);
    else
    if (Sender == tbtPicture)
      fpMain->CppCreatingControlClass = (unsigned int) __classid(TFlexPicture);
    else
    if (!((TToolButton *)Sender)->Tag)
      fpMain->CppCreatingControlClass = ((TToolButton *)Sender)->Tag;
    //if ToolMode in [ftmZoom, ftmZooming, ftmPan, ftmPanning] then
    fpMain->ToolMode = ftmSelect;
  }
}
//---------------------------------------------------------------------------
bool __fastcall TfmMain::CheckSave()
{
  if (!fpMain->Modified) return true;
  int ModalRes = MessageDlg("Save changes?", mtWarning,
    TMsgDlgButtons() << mbYes << mbNo << mbCancel, 0);
  switch (ModalRes) {
    case mrYes:
      {
        acFileSave->Execute();
        if (fpMain->Modified) return false;
        break;
      }
   case mrNo:
      break;
   case mrCancel:
      return false;
  }
  return true;
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::CheckTools()
{
 // Flags
 bool IsSel = fpMain->SelectedCount > 0;
 // File
 acFileNew->Enabled = true;
 acFileOpen->Enabled = true;
 acFileSave->Enabled = fpMain->Modified;
 // Edit
 acEditDelete->Enabled = IsSel;
 acEditCut->Enabled = IsSel;
 acEditCopy->Enabled = IsSel;
 // Schemes
 acSchemeDelete->Enabled = fpMain->Schemes->Count > 1;
 // Status
 sbrMain->Panels->Items[sbpSelected]->Text =
   "Selected: " + IntToStr(fpMain->SelectedCount);
 sbrMain->Panels->Items[sbpScale]->Text =
   "Scale: " + IntToStr(fpMain->Scale);
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::DocChange()
{
  int i;
  // Update scheme tabs
  tstSchemes->Tabs->BeginUpdate();
  try
  {
    tstSchemes->Tabs->Clear();
    for (i = 0; i < fpMain->Schemes->Count; i++)
      tstSchemes->Tabs->Add(fpMain->Schemes->Controls[i]->Name);
    tstSchemes->TabIndex = fpMain->Schemes->IndexOf(fpMain->ActiveScheme);
  }
  __finally
  {
    tstSchemes->Tabs->EndUpdate();
  };
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::acFileNewExecute(TObject *Sender)
{
  if ( !CheckSave() ) return;
  fpMain->NewDocument();
  fpMain->Modified = false;
  CheckTools();
  DocChange();
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::acFileOpenExecute(TObject *Sender)
{
  if (!CheckSave() || !od_Main->Execute()) return;
  fpMain->LoadFromFile(od_Main->FileName);
  FFilename = od_Main->FileName;
  sd_Main->FileName = od_Main->FileName;
  CheckTools();
  DocChange();
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::acFileSaveExecute(TObject *Sender)
{
  //
  if (!sd_Main->Execute()) return;
  fpMain->SaveToFile(sd_Main->FileName);
  FFilename = sd_Main->FileName;
  od_Main->FileName = sd_Main->FileName;
  CheckTools();
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::acFileExitExecute(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::acEditCutExecute(TObject *Sender)
{
  fpMain->CutToClipboard();
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::acEditCopyExecute(TObject *Sender)
{
  fpMain->CopyToClipboard();
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::acEditPasteExecute(TObject *Sender)
{
  fpMain->PasteFromClipboard(FALSE);
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::acEditDeleteExecute(TObject *Sender)
{
  fpMain->DeleteSelected();
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::acZoomInExecute(TObject *Sender)
{
  fpMain->Zoom(fpMain->Scale * 2, NULL);
  CheckTools();
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::acZoomOutExecute(TObject *Sender)
{
  fpMain->Zoom(fpMain->Scale / 2, NULL);
  CheckTools();
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::acZoom100Execute(TObject *Sender)
{
  fpMain->Zoom(100, NULL);
  CheckTools();   
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::fpMainNotify(TObject *Sender,
      TFlexControl *Control, TFlexNotify Notify)
{
  if (ComponentState.Contains(csDestroying)) return; 
  switch (Notify) {
    case fnSelect:
    case fnScale: {
      CheckTools();
      break;
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::acSchemeNewExecute(TObject *Sender)
{
  // We strongly recommend to use CppCreate function to create flex-control
  // in C++Builder environment instead of native object creation via
  // Control = new TFlexControl(...). It prevents from some incompatibility in
  // object initialization section in Delphi and C++Builder. You also can use
  // TFlexPanel::CreateControl() method to create flex-controls in definite flex
  // panel.
  TFlexScheme *Scheme = (TFlexScheme *) TFlexScheme::CppCreate(
    __classid(TFlexScheme), fpMain, fpMain->Schemes, NULL);
  // new TFlexScheme(fpMain, fpMain->Schemes, NULL);
  AnsiString AName = Scheme->Name;
  if (InputQuery("Enter new scheme name", "Name: ", AName))
  {
    Scheme->Name = AName;
    DocChange();
    tstSchemes->TabIndex = fpMain->Schemes->IndexOf(Scheme);
    CheckTools();
  }
  else
    Scheme->Free();
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::acSchemeDeleteExecute(TObject *Sender)
{
  AnsiString AName = fpMain->Schemes->Controls[tstSchemes->TabIndex]->Name;
  if (MessageDlg("Delete scheme \"" + AName + "\"?", mtConfirmation,
    TMsgDlgButtons() << mbOK << mbCancel, 0) == mrOk)
  {
    fpMain->Schemes->Controls[tstSchemes->TabIndex]->Free();
    DocChange();
    CheckTools();
  }
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::acSchemeRenameExecute(TObject *Sender)
{
  AnsiString AName = fpMain->Schemes->Controls[tstSchemes->TabIndex]->Name;
  if (InputQuery("Enter scheme name", "Name: ", AName))
  {
    fpMain->Schemes->Controls[tstSchemes->TabIndex]->Name = AName;
    DocChange();
    CheckTools();
  }
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::tstSchemesChange(TObject *Sender, int NewTab,
      bool &AllowChange)
{
  fpMain->ActiveScheme = fpMain->Schemes->Controls[NewTab];
  AllowChange = true;
}
//---------------------------------------------------------------------------


