unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  FlexBase, FlexControls, FlexUtils, FlexProps, FlexPath, Buttons, ExtCtrls,
  ComCtrls, ToolWin;

type
  TfmMain = class(TForm)
    fpMain: TFlexPanel;
    ToolBar1: TToolBar;
    tbtLinkEdit: TToolButton;
    ToolButton2: TToolButton;
    tbtAlways: TToolButton;
    tbtAsNeeded: TToolButton;
    tbtNever: TToolButton;
    ToolButton6: TToolButton;
    tbtExit: TToolButton;
    tbtLinkCreate: TToolButton;
    tbtSelect: TToolButton;
    tbtDelete: TToolButton;
    ToolButton3: TToolButton;
    procedure FormCreate(Sender: TObject);
    procedure tbtAlwaysClick(Sender: TObject);
    procedure tbtAsNeededClick(Sender: TObject);
    procedure tbtNeverClick(Sender: TObject);
    procedure tbtExitClick(Sender: TObject);
    procedure fpMainNotify(Sender: TObject; Control: TFlexControl;
      Notify: TFlexNotify);
    procedure tbtToolClick(Sender: TObject);
    procedure fpMainPropChanged(Sender: TObject; Prop: TCustomProp);
    procedure tbtDeleteClick(Sender: TObject);
    procedure fpMainControlCreate(Sender: TObject; Control: TFlexControl;
      var ControlDocRect: TRect);
  private
    FConnector: TFlexConnector;
    FLinkEditing: boolean;
    procedure GenerateDiagram;
    procedure UpdateButtons;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  fmMain: TfmMain;

implementation

{$R *.DFM}

procedure SetConnectorStyle(Connector: TFlexConnector);
begin
 // Set connector pen style
 Connector.PenProp.Color := clNavy;
 // Set arrow begin cap (for first link point)
 with Connector.BeginCapProp do begin
  CapStyle := psTriangle;
  FixedOutlineColor := true;
  OutlineColor := clBlue;
  FixedFillColor := true;
  FillColor := clBlue;
 end;
 // Set arrow end cap (for second link point)
 with Connector.EndCapProp do begin
  CapStyle := psTriangle;
  FixedOutlineColor := true;
  OutlineColor := clBlue;
  FixedFillColor := true;
  FillColor := clBlue;
 end;
end;

// TfmMain ////////////////////////////////////////////////////////////////////

procedure TfmMain.FormCreate(Sender: TObject);
begin
 LoadFlexCursors;
 GenerateDiagram;
 fpMain.InDesign := true;
 UpdateButtons;
end;

procedure TfmMain.GenerateDiagram;
var Box: array[1..5] of TFlexBox;

 function CreateLink(LinkA: TFlexControl; Ax, Ay: integer;
   LinkB: TFlexControl; Bx, By: integer): TFlexConnector;
 begin
  // Create connector controls
  Result := TFlexConnector.Create(fpMain, fpMain.ActiveScheme,
    fpMain.ActiveLayer);
  // Set first link control with link point coordinates
  with Result.LinkAProp do begin
   LinkedControl := LinkA;
   LinkPoint := Point(Ax, Ay);
  end;
  // Set second link control with link point coordinates
  with Result.LinkBProp do begin
   LinkedControl := LinkB;
   LinkPoint := Point(Bx, By);
  end;
  SetConnectorStyle(Result);
 end;

 function CreateBox(Left, Top, Right, Bottom: integer; Color: TColor): TFlexBox;
 begin
  // Create rectangle in given place
  Result := TFlexBox.Create(fpMain, fpMain.ActiveScheme, fpMain.ActiveLayer);
  Result.DocRect := Rect(Left, Top, Right, Bottom);
  // Set fill color and default roundness
  Result.BrushProp.Color := Color;
  Result.RoundnessProp.Value := 10000;
 end;

begin
 LoadFlexCursors;
 with fpMain do begin
  ConnectorsMinGap := 12000;
  // Create 5 blocks
  Box[1] := CreateBox(120000, 20000, 200000, 60000, clRed);
  Box[2] := CreateBox(40000, 100000, 120000, 140000, clGreen);
  Box[3] := CreateBox(200000, 100000, 280000, 140000, clGreen);
  Box[4] := CreateBox(40000, 180000, 120000, 220000, clBlue);
  Box[5] := CreateBox(200000, 180000, 280000, 220000, clBlue);
  // Create links
  CreateLink(Box[1], 120000,  40000, Box[2],  80000, 100000);
  CreateLink(Box[1], 200000,  40000, Box[3], 240000, 100000);
  CreateLink(Box[2],  80000, 140000, Box[4],  80000, 180000);
  CreateLink(Box[3], 240000, 140000, Box[5], 240000, 180000);
  CreateLink(Box[4], 120000, 200000, Box[5], 200000, 200000);
 end;
end;

procedure TfmMain.fpMainNotify(Sender: TObject; Control: TFlexControl;
  Notify: TFlexNotify);
begin
 // Ff form is about to be destroyed or notify is other then fnSelect then exit
 if (csDestroying in ComponentState) or (Notify <> fnSelect) then exit;
 // If selected one control and it control is TFlexConnector then
 // store it in FConnector field else set FConnector = Nil
 if (fpMain.SelectedCount = 1) and (fpMain.Selected[0] is TFlexConnector)
  then FConnector := TFlexConnector(fpMain.Selected[0])
  else FConnector := Nil;
 // When FConnector changes we must update toolbar buttons
 UpdateButtons;
end;

procedure TfmMain.fpMainControlCreate(Sender: TObject; Control: TFlexControl;
  var ControlDocRect: TRect);
begin
 if Control is TFlexConnector then
  // Set default connector style
  SetConnectorStyle(Control as TFlexConnector);
end;

procedure TfmMain.fpMainPropChanged(Sender: TObject; Prop: TCustomProp);
begin
 // Update "reroute" buttons when RerouteMode property changed
 if Assigned(FConnector) and (Sender = FConnector) and
   (Prop = FConnector.RerouteModeProp) then UpdateButtons;
end;

procedure TfmMain.UpdateButtons;
var Btn: TToolButton;
    Creating: boolean;
begin
 // Set buttons state
 Creating := Assigned(fpMain.CreatingControlClass);
 tbtSelect.Down := (fpMain.ToolMode = ftmSelect) and not Creating;
 tbtLinkCreate.Down := (fpMain.ToolMode = ftmSelect) and Creating;
 tbtLinkEdit.Down := FLinkEditing;
 tbtAlways.Enabled := Assigned(FConnector);
 tbtAsNeeded.Enabled := Assigned(FConnector);
 tbtNever.Enabled := Assigned(FConnector);
 // Show rerouted mode for selected connector (set corresponding button)
 Btn := Nil;
 if Assigned(FConnector) then
  case FConnector.RerouteMode of
   rmAlways    : Btn := tbtAlways;
   rmAsNeeded  : Btn := tbtAsNeeded;
   rmNever     : Btn := tbtNever;
  end;
 tbtAlways.Down    := Btn = tbtAlways;
 tbtAsNeeded.Down  := Btn = tbtAsNeeded;
 tbtNever.Down     := Btn = tbtNever;
end;

procedure TfmMain.tbtToolClick(Sender: TObject);
begin
 FLinkEditing := tbtLinkEdit.Down;
 // Switch between selecting and curve editind modes
 if tbtLinkCreate.Down then begin
  fpMain.ToolMode := ftmSelect;
  fpMain.CreatingControlClass := TFlexConnector;
 end else begin
  fpMain.CreatingControlClass := Nil;
  if FLinkEditing
   then fpMain.ToolMode := ftmPointEdit
   else fpMain.ToolMode := ftmSelect;
 end;
end;

procedure TfmMain.tbtDeleteClick(Sender: TObject);
begin
 fpMain.DeleteSelected;
end;

procedure TfmMain.tbtAlwaysClick(Sender: TObject);
begin
 if not Assigned(FConnector) then exit;
 FConnector.RerouteMode := rmAlways;
end;

procedure TfmMain.tbtAsNeededClick(Sender: TObject);
begin
 if not Assigned(FConnector) then exit;
 FConnector.RerouteMode := rmAsNeeded;
end;

procedure TfmMain.tbtNeverClick(Sender: TObject);
begin
 if not Assigned(FConnector) then exit;
 FConnector.RerouteMode := rmNever;
end;

procedure TfmMain.tbtExitClick(Sender: TObject);
begin
 Close;
end;

end.
