unit MyCtrls;

interface

uses
  Graphics, Windows, SysUtils,
  FlexBase, FlexControls, FlexProps, MyProps;

type
  TMyComplexControl = class(TFlexControl)
  private
   FFontProp: TFontProp;
   FMyProp: TMyComplexProp;
  protected
   procedure CreateProperties; override;
   procedure ControlCreate; override;
   procedure Paint(Canvas: TCanvas; var PaintRect: TRect); override;
  public
   property  FontProp: TFontProp read FFontProp;
   property  MyProp: TMyComplexProp read FMyProp;
  end;

implementation

// TMyComplexControl //////////////////////////////////////////////////////////

procedure TMyComplexControl.CreateProperties;
begin
 inherited;
 FFontProp := TFontProp.Create(Props, 'Font');
 FMyProp := TMyComplexProp.Create(Props, 'MyComplex');
end;

procedure TMyComplexControl.ControlCreate;
begin
 Width := 1;
 Height := 1;
 inherited;
 Visible := True;
end;

procedure TMyComplexControl.Paint(Canvas: TCanvas; var PaintRect: TRect);
var Data: string;
    R: TRect;
begin
 FFontProp.Setup(Canvas, Owner.Scale);
 with Canvas, PaintRect do begin
  // Paint static background
  Brush.Color := clSilver;
  Brush.Style := bsSolid;
  Pen.Color := clBlack;
  Pen.Style := psSolid;
  Pen.Width := 1;
  Pen.Mode := pmCopy;
  {$IFDEF VER120}
  Rectangle(PaintRect.Left, PaintRect.Top, PaintRect.Right, PaintRect.Bottom);
  {$ELSE}
  Rectangle(PaintRect);
  {$ENDIF}
  R := PaintRect;
  InflateRect(R, -4, -4);
  // Paint value and caption
  Font.Style := Canvas.Font.Style + [ fsBold ];
  Data := Format('%d. %s', [FMyProp.Value, FMyProp.Caption]);
  DrawText(Handle, PChar(Data), Length(Data), R, DT_NOPREFIX or DT_SINGLELINE);
  Font.Style := Canvas.Font.Style - [ fsBold ];
  // Paint data
  FMyProp.Font.Setup(Canvas, Owner.Scale);
  Data := MyProp.Data.Strings.Text;
  inc(R.Top, 5 + Font.Size * Font.PixelsPerInch div 72);
  DrawText(Handle, PChar(Data), Length(Data), R, DT_NOPREFIX or DT_WORDBREAK);
 end;
end;

initialization
  RegisterFlexControl(TMyComplexControl);

end.
