//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Main.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "FlexBase"
#pragma resource "*.dfm"
TfmMain *fmMain;
//---------------------------------------------------------------------------
__fastcall TfmMain::TfmMain(TComponent* Owner)
        : TForm(Owner)
{
  // Load cursor for curve editing
  LoadFlexCursors();
  // Switch FlexPanel to InDesign mode
  fpView->InDesign = true;
}
//---------------------------------------------------------------------------

void __fastcall AnyKey(WideString Msg)
{
  MessageDlg(Msg + "\n\nConfirm to continue", mtInformation,
    TMsgDlgButtons() << mbOK, 0);
}

//---------------------------------------------------------------------------
void __fastcall TfmMain::btLoadClick(TObject *Sender)
{
  if (od_Main->Execute())
    fpView->LoadFromFile(od_Main->FileName);
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btSaveClick(TObject *Sender)
{
  if (sd_Main->Execute())
    fpView->SaveToFile(sd_Main->FileName);
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btDemoClick(TObject *Sender)
{
  // Deselect all currently selected objects
  fpView->UnselectAll();
  // Create box
   // We strongly recommend to use CppCreate function to create flex-control
   // in C++Builder environment instead of native object creation via
   // Control = new TFlexControl(...). It prevents from some incompatibility in
   // object initialization section in Delphi and C++Builder. You also can use
   // TFlexPanel::CreateControl() method to create flex-controls in definite flex
   // panel.
  TFlexBox* Box = (TFlexBox *) TFlexBox::CppCreate(fpView,
    fpView->ActiveScheme, fpView->ActiveLayer);
  //new TFlexBox(fpView, fpView->ActiveScheme, fpView->ActiveLayer);
  Box->Left = 10000;
  Box->Top = 20000;
  Box->Width = 200000;
  Box->Height = 100000;
  Box->BrushProp->Color = clLime;
  AnyKey("Box created");
  // Create ellipse
  TFlexEllipse* Ellipse = (TFlexEllipse *) TFlexEllipse::CppCreate(fpView,
    fpView->ActiveScheme, fpView->ActiveLayer);
  // new TFlexEllipse(fpView, fpView->ActiveScheme, fpView->ActiveLayer);
  Ellipse->Left = 50000;
  Ellipse->Top = 60000;
  Ellipse->Width = 200000;
  Ellipse->Height = 100000;
  Ellipse->BeginAngleProp->Value = 330000;
  Ellipse->EndAngleProp->Value = 270000;
  Ellipse->BrushProp->Color = clYellow;
  AnyKey("Ellipse created");
  // Create regular polygon
  TFlexRegularPolygon *Polygon = (TFlexRegularPolygon *) 
	TFlexRegularPolygon::CppCreate(fpView, fpView->ActiveScheme,
	fpView->ActiveLayer);
  Polygon->Left = 15000;
  Polygon->Top = 130000;
  Polygon->Width = 50000;
  Polygon->Height = 50000;
  Polygon->SidesProp->Value = 5;
  Polygon->BrushProp->Color = clFuchsia;
  AnyKey("Polygon created");
  // Create curve
  TFlexCurve* Curve = (TFlexCurve *) TFlexCurve::CppCreate(fpView,
    fpView->ActiveScheme, fpView->ActiveLayer);
  // new TFlexCurve(fpView, fpView->ActiveScheme, fpView->ActiveLayer);
  Curve->Left = 100000;
  Curve->Top = 70000;
  Curve->BeginUpdate();
  try
  {
    // Delete all points
    while (Curve->PointCount > 0) Curve->DeletePoint(0);
    // Create first figure (solid)
    Curve->AddPoint(Point(0, 20000));
    Curve->AddCurvePoints(Point(20000, 20000), Point(30000, 0), Point(50000, 40000));
    Curve->AddPoint(Point(60000, 20000));
    Curve->AddCurvePoints(Point(80000, 20000), Point(70000, 50000), Point(10000, 50000));
    Curve->AddPoint(Point(0, 20000));
    Curve->EndFigure(True);
    // Create second figure (non-solid)
    Curve->AddCurvePoints(Point(0, 20000), Point(10000, -10000), Point(70000, -10000));
    Curve->AddPoint(Point(80000, 20000));
    Curve->EndFigure(False);
    Curve->BrushProp->Color = clMaroon;
  }
  __finally
  {
   Curve->EndUpdate();
  }
  AnyKey("Curve created");
  // Create text
  TFlexText* Text = (TFlexText *) TFlexText::CppCreate(fpView,
    fpView->ActiveScheme, fpView->ActiveLayer);
  // new TFlexText(fpView, fpView->ActiveScheme, fpView->ActiveLayer);
  Text->Left = 10000;
  Text->Top = 190000;
  Text->Width = 300000;
  Text->Height = 40000;
  Text->Alignment = taCenter;
  Text->Layout = tlCenter;
  Text->TextProp->Text = "FlexGraphics Demo";
  Text->FontProp->Size = 22000;
  Text->FontProp->Style =
    System::Set<TFontStyle, fsBold, fsStrikeOut> () << fsBold; // [fsBold];
  Text->BrushProp->Method = bmGradient;
  Text->BrushProp->GradStyle = gsVertical;
  Text->BrushProp->GradBeginColor = clYellow;
  Text->BrushProp->GradEndColor = clOlive;
  AnyKey("Text created");
  // Create picture
  TFlexPicture* Picture = (TFlexPicture *) TFlexPicture::CppCreate(fpView,
    fpView->ActiveScheme, fpView->ActiveLayer);
  // new TFlexPicture(fpView, fpView->ActiveScheme, fpView->ActiveLayer);
  Picture->Left = 260000;
  Picture->Top = 20000;
  Graphics::TBitmap* Bmp = new Graphics::TBitmap();
  try
  {
   // Load from resource
   Bmp->LoadFromResourceName((unsigned int)HInstance, "BITMAP1");
   Picture->PictureProp->Graphic = Bmp;
   // Set exact width and height
   Picture->AutoSizeProp->Value = true;
   Picture->AutoSizeProp->Value = false;
   // Set transparent
   Picture->PictureProp->Masked = true;
  }
  __finally
  {
   delete Bmp;
  }
  // Select box and ellipse objects
  fpView->Select(Box);
  fpView->Select(Ellipse);
  // Convert selected objects to curve objects
  fpView->ConvertSelectedToCurves();
  // Combine selected curves in one object
  fpView->CombineSelected();
  AnyKey("Box and ellipse selected, converted to curves and combined");
  // Define result combination object
  TFlexControl* Comb = fpView->Selected[0];
  // Find 'Brush' flex-property
  TBrushProp* CombBrush = (TBrushProp*)Comb->Props->Props["Brush"];
  if (CombBrush != NULL)
  {
    // Change filling to gradient method
    CombBrush->GradBeginColor = clNavy;
    CombBrush->GradEndColor = clAqua;
    CombBrush->GradStyle = gsVertical;
    CombBrush->Method = bmGradient;
  }
  AnyKey("Object fill changed to gradient");
  // Deselect all objects
  fpView->UnselectAll();
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::sbtPointEditClick(TObject *Sender)
{
  if (sbtPointEdit->Down)
    fpView->ToolMode = ftmPointEdit;
  else
    fpView->ToolMode = ftmSelect;
}
//---------------------------------------------------------------------------


