unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  FlexBase, FlexControls, FlexProps, FlexUtils, StdCtrls, ExtCtrls, Buttons;

type
  TForm1 = class(TForm)
    FlexPanel1: TFlexPanel;
    od_Main: TOpenDialog;
    sd_Main: TSaveDialog;
    Panel2: TPanel;
    btDemo: TButton;
    btLoad: TButton;
    btSave: TButton;
    sbtPointEdit: TSpeedButton;
    Bevel1: TBevel;
    Bevel2: TBevel;
    procedure btDemoClick(Sender: TObject);
    procedure btLoadClick(Sender: TObject);
    procedure btSaveClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure sbtPointEditClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
begin
 // Switch FlexPanel to InDesign mode
 FlexPanel1.InDesign := true;
 // Load cursor for curve editing
 LoadFlexCursors;
end;

procedure TForm1.btDemoClick(Sender: TObject);
var Box: TFlexBox;
    Ellipse: TFlexEllipse;
    Polygon: TFlexRegularPolygon;
    Curve: TFlexCurve;
    Text: TFlexText;
    Picture: TFlexPicture;
    Comb: TFlexControl;
    CombBrush: TBrushProp;
    Bmp: TBitmap;

 procedure AnyKey(const Msg: string);
 begin
  MessageDlg(Msg+#13#10#13#10'Confirm to continue', mtInformation, [mbOk], 0);
 end;

begin
 with FlexPanel1 do begin
  // Deselect all currently selected objects
  UnselectAll;
  // Create box
  Box := TFlexBox.Create(FlexPanel1, ActiveScheme, ActiveLayer);
  Box.Left := 10000;
  Box.Top := 20000;
  Box.Width := 200000;
  Box.Height := 100000;
  Box.BrushProp.Color := clLime;
  AnyKey('Box created');
  // Create ellipse
  Ellipse := TFlexEllipse.Create(FlexPanel1, ActiveScheme, ActiveLayer);
  Ellipse.Left := 70000;
  Ellipse.Top := 60000;
  Ellipse.Width := 200000;
  Ellipse.Height := 100000;
  Ellipse.BeginAngleProp.Value := 330000;
  Ellipse.EndAngleProp.Value := 270000;
  Ellipse.BrushProp.Color := clYellow;
  AnyKey('Ellipse created');
  // Create regular polygon
  Polygon := TFlexRegularPolygon.Create(FlexPanel1, ActiveScheme, ActiveLayer);
  Polygon.Left := 15000;
  Polygon.Top := 130000;
  Polygon.Width := 50000;
  Polygon.Height := 50000;
  Polygon.SidesProp.Value := 5;
  Polygon.BrushProp.Color := clFuchsia;
  AnyKey('Polygon created');
  // Create curve
  Curve := TFlexCurve.Create(FlexPanel1, ActiveScheme, ActiveLayer);
  Curve.Left := 100000;
  Curve.Top := 70000;
  Curve.BeginUpdate;
  with Curve do
  try
   // Delete all points
   while PointCount > 0 do DeletePoint(0);
   // Create first figure (solid)
   AddPoint(Point(0, 20000));
   AddCurvePoints(Point(20000, 20000), Point(30000, 0), Point(50000, 40000));
   AddPoint(Point(60000, 20000));
   AddCurvePoints(Point(80000, 20000), Point(70000, 50000), Point(10000, 50000));
   AddPoint(Point(0, 20000));
   EndFigure(True);
   // Create second figure (non-solid)
   AddCurvePoints(Point(0, 20000), Point(10000, -10000), Point(70000, -10000));
   AddPoint(Point(80000, 20000));
   EndFigure(False);
   BrushProp.Color := clMaroon;
  finally
   EndUpdate;
  end;
  AnyKey('Curve created');
  // Create text
  Text := TFlexText.Create(FlexPanel1, ActiveScheme, ActiveLayer);
  Text.Left := 10000;
  Text.Top := 190000;
  Text.Width := 300000;
  Text.Height := 40000;
  Text.Alignment := taCenter;
  Text.Layout := tlCenter;
  Text.TextProp.Text := 'FlexGraphics Demo';
  Text.FontProp.Size := 22000;
  Text.FontProp.Style := [fsBold];
  with Text.BrushProp do begin
   Method := bmGradient;
   GradStyle := gsVertical;
   GradBeginColor := clYellow;
   GradEndColor := clOlive;
  end;
  AnyKey('Text created');
  // Create picture
  Picture := TFlexPicture.Create(FlexPanel1, ActiveScheme, ActiveLayer);
  Picture.Left := 260000;
  Picture.Top := 20000;
  Bmp := TBitmap.Create;
  try
   // Load from resource
   Bmp.LoadFromResourceName(HInstance, 'BITMAP1');
   Picture.PictureProp.Graphic := Bmp;
   // Set exact width and height
   Picture.AutoSizeProp.Value := true;
   Picture.AutoSizeProp.Value := false;
   // Set transparent
   Picture.PictureProp.Masked := true;
  finally
   Bmp.Free;
  end;
  AnyKey('Picture created');
  // Select box and ellipse objects
  Select(Box);
  Select(Ellipse);
  // Convert selected objects to curve objects
  ConvertSelectedToCurves;
  // Combine selected curves in one object
  CombineSelected;
  AnyKey('Box and ellipse selected, converted to curves and combined');
  // Define result combination object
  Comb := Selected[0];
  // Find 'Brush' flex-property
  CombBrush := TBrushProp(Comb.Props['Brush']);
  if Assigned(CombBrush) then with CombBrush do begin
   // Change filling to gradient method
   GradBeginColor := clNavy;
   GradEndColor := clAqua;
   GradStyle := gsVertical;
   Method := bmGradient;
   AnyKey('Object fill changed to gradient');
  end;
  // Deselect all objects
  UnselectAll;
 end;
end;

procedure TForm1.btLoadClick(Sender: TObject);
begin
 if od_Main.Execute then FlexPanel1.LoadFromFile(od_Main.FileName);
end;

procedure TForm1.btSaveClick(Sender: TObject);
begin
 if sd_Main.Execute then FlexPanel1.SaveToFile(sd_Main.FileName);
end;

procedure TForm1.sbtPointEditClick(Sender: TObject);
begin
 if sbtPointEdit.Down
  then FlexPanel1.ToolMode := ftmPointEdit
  else FlexPanel1.ToolMode := ftmSelect;
end;

end.
