unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ToolWin, ComCtrls, FlexBase, ImgList, FlexUtils, FlexProps, GraphControls;

type
  TfmMain = class(TForm)
    fpMain: TFlexPanel;
    tbrMain: TToolBar;
    tbtPick: TToolButton;
    tbtZoom: TToolButton;
    tbtPan: TToolButton;
    tbtConnect: TToolButton;
    imgTools: TImageList;
    procedure FormCreate(Sender: TObject);
    procedure tbtPickClick(Sender: TObject);
    procedure tbtZoomClick(Sender: TObject);
    procedure tbtPanClick(Sender: TObject);
    procedure tbtConnectClick(Sender: TObject);
    procedure fpMainMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure fpMainMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    { Private declarations }
    // Layers
    FMarkerLayer: TFlexLayer;
    FNodeLayer: TFlexLayer;
    FLineLayer: TFlexLayer;
    // Marker controls
    FMarkerNode: TGraphNode;
    FDesignLine: TGraphLine;
    // Highlighting
    FHighlightedNode: TGraphNode;
    FHighlightedLine: TGraphLine;
    procedure CheckTools;
    procedure UpdateLayers;
  public
    { Public declarations }
  end;

var
  fmMain: TfmMain;

implementation

{$R *.DFM}

function FindByName(FlexPanel: TFlexPanel; const FirstChars: string;
  TopControl: TFlexControl = Nil): TFlexControl;
var Control: TFlexControl;
    PassRec: TPassControlRec;
    TestStr, NameStr: string;
begin
 Result := Nil;
 if Assigned(TopControl)
  then Control := TopControl
  else Control := FlexPanel.Schemes;
 TestStr := LowerCase(FirstChars);
 FirstControl(Control, PassRec);
 while Assigned(Control) do begin
  NameStr := LowerCase(Control.Name);
  if Pos(TestStr, NameStr) = 1 then begin
   Result := Control;
   break;
  end;
  Control := NextControl(PassRec);
 end;
 ClosePassRec(PassRec)
end;

procedure TfmMain.FormCreate(Sender: TObject);
begin
 LoadFlexCursors;
 // Create layers
 FNodeLayer := fpMain.Layers.New;
 FNodeLayer.Name := 'Nodes';
 FLineLayer := fpMain.Layers.New;
 FLineLayer.Name := 'Lines'; 
 FMarkerLayer := fpMain.Layers.New;
 FMarkerLayer.Name := 'Marker';
 // Create marker controls
 FMarkerNode := TGraphNode.Create(fpMain, fpMain.ActiveScheme, FMarkerLayer);
 FMarkerNode.Name := 'Marker Node';
 FMarkerNode.IsMarker := true;
 FMarkerNode.Visible := false;
 // Update tools
 CheckTools;
end;

procedure TfmMain.CheckTools;
begin
 UpdateLayers;
 // Check InDesign mode
 fpMain.InDesign := not tbtConnect.Down;
 // Check not finished line
 if Assigned(FDesignLine) and not tbtConnect.Down then FreeAndNil(FDesignLine);
end;

procedure TfmMain.UpdateLayers;
begin
 FMarkerLayer.Selectable := false;
 if tbtConnect.Down then begin
  FNodeLayer.Selectable := true;
  FLineLayer.Selectable := true;
 end else begin
  FNodeLayer.Selectable := true;
  FLineLayer.Selectable := false;
 end;
end;

procedure TfmMain.tbtPickClick(Sender: TObject);
begin
 fpMain.ToolMode := ftmSelect;
 CheckTools;
end;

procedure TfmMain.tbtZoomClick(Sender: TObject);
begin
 fpMain.ToolMode := ftmZoom;
 CheckTools;
end;

procedure TfmMain.tbtPanClick(Sender: TObject);
begin
 fpMain.ToolMode := ftmPan;
 CheckTools;
end;

procedure TfmMain.tbtConnectClick(Sender: TObject);
begin
 fpMain.UnselectAll;
 fpMain.ToolMode := ftmSelect;
 CheckTools;
end;

procedure TfmMain.fpMainMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
var MouseNode: TGraphNode;
    MouseLine: TGraphLine;
begin
 // Update marker position
 FMarkerNode.Center := fpMain.Schemes.OwnerToClient(Point(x, y));
 if tbtConnect.Down then begin
  MouseNode := Nil;
  MouseLine := Nil;
  // Define control under mouse cursor
  if Assigned(fpMain.MouseSubControl) then
   if fpMain.MouseSubControl is TGraphNode then
    MouseNode := TGraphNode(fpMain.MouseSubControl)
   else
   if fpMain.MouseSubControl is TGraphLine then
    MouseLine := TGraphLine(fpMain.MouseSubControl);
  // Update highlighted node
  if FHighlightedNode <> MouseNode then begin
   if Assigned(FHighlightedNode) then FHighlightedNode.Highlighted := false;
   FHighlightedNode := MouseNode;
   if Assigned(FHighlightedNode) then FHighlightedNode.Highlighted := true;
   // Update NodeB for DesignLine
   if Assigned(FDesignLine) then
    if Assigned(FHighlightedNode)
     then FDesignLine.NodeB := FHighlightedNode
     else FDesignLine.NodeB := FMarkerNode;
  end;
  // Update highlighted line
  if Assigned(FDesignLine) then MouseLine := Nil;
  if FHighlightedLine <> MouseLine then begin
   if Assigned(FHighlightedLine) then FHighlightedLine.ForDelete := false;
   FHighlightedLine := MouseLine;
   if Assigned(FHighlightedLine) then FHighlightedLine.ForDelete := true;
  end;
 end;
 // If mouse over existing node then hide marker node
 FMarkerNode.Visible := tbtConnect.Down and not Assigned(FHighlightedNode) and
   not Assigned(FHighlightedLine);
end;

procedure TfmMain.fpMainMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var Btn: TToolButton;
begin
 if Button = mbRight then begin
  Btn := Nil;
  if tbtConnect.Down then begin
   if not Assigned(FDesignLine) then Btn := tbtPick;
  end else begin
   fpMain.UnselectAll;
   Btn := tbtConnect;
  end;
  if Assigned(Btn) then begin
   Btn.Down := true;
   Btn.Click;
  end;
 end else
 if tbtConnect.Down and (Button = mbLeft) then begin
  if Assigned(FHighlightedLine) then begin
   FreeAndNil(FHighlightedLine);  
   exit;
  end;
  // Create or use existing node
  if not Assigned(FHighlightedNode) then begin
   // Create new node
   FHighlightedNode := TGraphNode.Create(fpMain, fpMain.ActiveScheme, FNodeLayer);
   FHighlightedNode.Center := FMarkerNode.Center;
   FHighlightedNode.Highlighted := true;
   FMarkerNode.Visible := false;
  end;
  // Create or use existing line
  if not Assigned(FDesignLine) then begin
   FDesignLine := TGraphLine.Create(fpMain, fpMain.ActiveScheme, FMarkerLayer);
   FDesignLine.Designing := true;
   // Connect NodeA to highlighted node
   FDesignLine.NodeA := FHighlightedNode;
   // Connect NodeB to marker
   FDesignLine.NodeB := FMarkerNode;
   with FDesignLine.EndCapProp do begin
    CapStyle := psTriangle;
    FixedOutlineColor := true;
    OutlineColor := clNone;
   end;
   FLineLayer.Selectable := false;
  end else begin
   // Set NodeB to highlighted
   FDesignLine.NodeB := FHighlightedNode;
   FDesignLine.Layer := FLineLayer;
   FDesignLine.Designing := false;
   FDesignLine := Nil;
   FLineLayer.Selectable := true;
  end;
 end;
end;

end.
