unit Main;

interface

// Enable this define if you have installed RX Library
{.DEFINE USE_RXGIF}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  {$IFDEF USE_RXGIF}
  RXGif, // rigister GIF format in Delphi Graphics module
  {$ENDIF}
  Menus, ExtDlgs, ImgList, ComCtrls, StdCtrls, ExtCtrls, Buttons, ToolWin,
  FlexBase, FlexProps, FlexControls, FlexUtils, Layers;

type
  TfmMain = class(TForm)
    mmMain: TMainMenu;
    miFile: TMenuItem;
    miExit: TMenuItem;
    fpMain: TFlexPanel;
    miControls: TMenuItem;
    miScheme: TMenuItem;
    miSchemeLoadImage: TMenuItem;
    miSchemeClearImage: TMenuItem;
    miSchemeLoadTexture: TMenuItem;
    miSchemeClearTexture: TMenuItem;
    N2: TMenuItem;
    miSchemeStretch: TMenuItem;
    miControlsCreatePicture: TMenuItem;
    opd_Main: TOpenPictureDialog;
    miControlsDelete: TMenuItem;
    miControlsInverseTrasparent: TMenuItem;
    N1: TMenuItem;
    N3: TMenuItem;
    miControlsGenerate100: TMenuItem;
    imgLayerImages: TImageList;
    panBottom: TPanel;
    Label1: TLabel;
    trbResize: TTrackBar;
    lblResizePercent: TLabel;
    miControlsRoateSelected: TMenuItem;
    Bevel1: TBevel;
    Label2: TLabel;
    trbScale: TTrackBar;
    lblScale: TLabel;
    sbtScale100: TSpeedButton;
    imgTools: TImageList;
    tbrMain: TToolBar;
    tbtPick: TToolButton;
    tbtZoom: TToolButton;
    tbtPan: TToolButton;
    panLayers: TPanel;
    splRight: TSplitter;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure miExitClick(Sender: TObject);
    procedure miSchemeLoadImageClick(Sender: TObject);
    procedure miSchemeLoadTextureClick(Sender: TObject);
    procedure miSchemeClearImageClick(Sender: TObject);
    procedure miSchemeClearTextureClick(Sender: TObject);
    procedure miSchemeStretchClick(Sender: TObject);
    procedure miControlsCreatePictureClick(Sender: TObject);
    procedure miControlsDeleteClick(Sender: TObject);
    procedure miControlsInverseTrasparentClick(Sender: TObject);
    procedure miControlsGenerate100Click(Sender: TObject);
    procedure miControlsRoateSelectedClick(Sender: TObject);
    procedure sbtScale100Click(Sender: TObject);
    procedure tbtPickClick(Sender: TObject);
    procedure tbtZoomClick(Sender: TObject);
    procedure tbtPanClick(Sender: TObject);
    procedure trbResizeChange(Sender: TObject);
    procedure trbScaleChange(Sender: TObject);
    procedure fpMainNotify(Sender: TObject; Control: TFlexControl;
      Notify: TFlexNotify);
  private
    { Private declarations }
    FPicture: TPicture;
    FResizable: boolean;
    FResizeUpdating: boolean;
    FLayers: TfmLayers;
    function  CreatePicture(Layer: TFlexLayer;
      PictureWidth, PictureHeight: integer): TFlexPicture;
    procedure UpdateResizePanel;
  public
    { Public declarations }
  end;

var
  fmMain: TfmMain;

implementation

{$R *.DFM}

procedure TfmMain.FormCreate(Sender: TObject);
var
  R: TRect;
begin
 LoadFlexCursors;
 Randomize;
 FPicture := TPicture.Create;
 // Center scroll area
 R := fpMain.ActiveScheme.DocRect;
 fpMain.Zoom(100, @R);
 // Generate additional 9 layers
 while fpMain.Layers.Count < 10 do fpMain.Layers.New;
 // Enables selecting, resizing etc.
 fpMain.InDesign := true;
 // Generate 100 images on different layers
 miControlsGenerate100.Click;
 // Embed the layer manager into main form
 FLayers := TfmLayers.Create(Self);
 FLayers.Parent := panLayers;
 FLayers.Align := alClient;
 FLayers.BorderStyle := bsNone;
 FLayers.Visible := True;
end;

procedure TfmMain.FormDestroy(Sender: TObject);
begin
 FPicture.Free;
end;

function TfmMain.CreatePicture(Layer: TFlexLayer;
  PictureWidth, PictureHeight: integer): TFlexPicture;
var R, RDoc: TRect;
    Pos: TPoint;
begin
 // Generate pos within fpMain.ClientRect
 R := fpMain.ClientRect;
 with fpMain, ActiveScheme, Schemes.DocRect do begin
  RDoc.TopLeft := ClientToOwner(TopLeft);
  RDoc.BottomRight := ClientToOwner(BottomRight);
 end;
 IntersectRect(R, R, RDoc);
 Pos.X := Round(Random * (R.Right - R.Left - PictureWidth));
 Pos.Y := Round(Random * (R.Bottom - R.Top - PictureHeight));
 // Convert to document coordinates
 fpMain.UnTransformPoint(Pos.X, Pos.Y);
 R := Rect(Pos.X, Pos.Y,
   Pos.X + PictureWidth * PixelScaleFactor,
   Pos.Y + PictureHeight * PixelScaleFactor);
 // Create FlexPicture control
 Result := TFlexPicture.Create(fpMain, fpMain.ActiveScheme, Layer);
 // Set position
 Result.DocRect := R;
end;

procedure TfmMain.miExitClick(Sender: TObject);
begin
 Close;
end;

procedure TfmMain.miSchemeLoadImageClick(Sender: TObject);
begin
 if opd_Main.Execute then begin
  // Load picture
  FPicture.LoadFromFile(opd_Main.FileName);
  // Assign loaded picture to scheme background picture property
  TFlexScheme(fpMain.ActiveScheme).PictureProp.Graphic := FPicture.Graphic;
  FPicture.Graphic := Nil;
 end;
end;

procedure TfmMain.miSchemeLoadTextureClick(Sender: TObject);
var Bmp: TBitmap;
    Filter: string;
begin
 Filter := opd_Main.Filter;
 opd_Main.Filter := 'Bitmaps (*.bmp)|*.bmp|All files (*.*)|*.*';
 if opd_Main.Execute then begin
  // Texture must be bitmap
  Bmp := TBitmap.Create;
  Bmp.LoadFromFile(opd_Main.FileName);
  with TFlexScheme(fpMain.ActiveScheme).BrushProp do begin
   // Set texture picture
   Bitmap := Bmp;
   // Set texture paint method
   Method := bmBitmap;
  end;
  // Do not destroy Bmp!
 end;
 // Loaded texture will be painted only if scheme PictureProp is cleared!
 opd_Main.Filter := Filter;
end;

procedure TfmMain.miSchemeClearImageClick(Sender: TObject);
begin
 // Clear scheme background picture
 TFlexScheme(fpMain.ActiveScheme).PictureProp.Graphic := Nil;
end;

procedure TfmMain.miSchemeClearTextureClick(Sender: TObject);
begin
 // Clear scheme brush texture
 with TFlexScheme(fpMain.ActiveScheme).BrushProp do begin
  Bitmap := Nil;
  Method := bmHatch;
 end;
end;

procedure TfmMain.miSchemeStretchClick(Sender: TObject);
begin
 // On/Off scheme background stretching
 miSchemeStretch.Checked := not miSchemeStretch.Checked;
 fpMain.SchemeBkStretch := miSchemeStretch.Checked;
end;

procedure TfmMain.miControlsCreatePictureClick(Sender: TObject);
begin
 if not opd_Main.Execute then exit;
 // Load picture
 FPicture.LoadFromFile(opd_Main.FileName);
 // Create FlexPicture control on topmost layer and assign loaded image
 with CreatePicture(fpMain.Layers[fpMain.Layers.Count-1],
   FPicture.Width, FPicture.Height) do begin
  PictureProp.Graphic := FPicture.Graphic;
  fpMain.UnselectAll;
  IsSelected := True;
 end;
 FPicture.Graphic := Nil;
end;

procedure TfmMain.miControlsDeleteClick(Sender: TObject);
begin
 fpMain.DeleteSelected;
end;

procedure TfmMain.miControlsInverseTrasparentClick(Sender: TObject);
var i: integer;
begin
 // Works correct only for bitmaps.
 for i:=0 to fpMain.SelectedCount-1 do
  if fpMain.Selected[i] is TFlexPicture then
   with TFlexPicture(fpMain.Selected[i]).PictureProp do Masked := not Masked;
end;

procedure TfmMain.miControlsGenerate100Click(Sender: TObject);
var i, LayerIndex: integer;
    Bmp: TBitmap;
begin
 for i:=1 to 100 do begin
  // Select random layer
  LayerIndex := Round(Random*(fpMain.Layers.Count-1));
  // Create FlexPicture control on this layer
  with CreatePicture(fpMain.Layers[LayerIndex],
    imgLayerImages.Width, imgLayerImages.Height) do
   if LayerIndex < imgLayerImages.Count then begin
    // Assign layer image
    Bmp := TBitmap.Create;
    try
     imgLayerImages.GetBitmap(LayerIndex, Bmp);
     Bmp.Transparent := false;
     PictureProp.Graphic := Bmp;
     AutoSizeProp.Value := true;
     AutoSizeProp.Value := false;
    finally
     Bmp.Free;
    end;
   end;
 end;
end;

procedure TfmMain.fpMainNotify(Sender: TObject; Control: TFlexControl;
  Notify: TFlexNotify);
begin
 if csDestroying in ComponentState then exit;
 case Notify of
  fnSelect:
    begin
     // Check resizability of selected control
     FResizable :=
       (fpMain.SelectedCount = 1) and (fpMain.Selected[0] is TFlexPicture) and
       TFlexPicture(fpMain.Selected[0]).PictureProp.IsLoaded;
     UpdateResizePanel;
    end;
  fnRect:
    // Update resize panel when control resizing manualy
    if FResizable and not FResizeUpdating then UpdateResizePanel;
  fnScale:
    begin
      trbScale.Position := fpMain.Scale;
      lblScale.Caption := Format('%d%%', [trbScale.Position]);
    end;
 end;
end;

procedure TfmMain.UpdateResizePanel;
var CoeffX, CoeffY: double;
begin
 FResizeUpdating := true;
 try
  trbResize.Enabled := FResizable;
  lblResizePercent.Enabled := FResizable;
  if FResizable then with TFlexPicture(fpMain.Selected[0]) do begin
   // Define minimal scaling factor
   CoeffX := UnscalePixels(Width) / PictureProp.Graphic.Width;
   CoeffY := UnscalePixels(Height) / PictureProp.Graphic.Height;
   if CoeffY < CoeffX then CoeffX := CoeffY;
   // Update track bar position
   trbResize.Position := Round(CoeffX * 100);
  end;
 finally
  FResizeUpdating := false;
 end;
end;

procedure TfmMain.trbResizeChange(Sender: TObject);
begin
 lblResizePercent.Caption := IntToStr(trbResize.Position)+'%';
 if FResizeUpdating or not FResizable then exit;
 with TFlexPicture(fpMain.Selected[0]) do begin
  // Update selected FlexPicture size
  BeginUpdate;
  try
   FResizeUpdating := true;
   Width := ScalePixels(
     Round(trbResize.Position / 100 * PictureProp.Graphic.Width) );
   Height := ScalePixels(
     Round(trbResize.Position / 100 * PictureProp.Graphic.Height) );
  finally
   EndUpdate;
   FResizeUpdating := false;
  end;
 end;
end;

procedure TfmMain.miControlsRoateSelectedClick(Sender: TObject);
var i: integer;
    Control: TFlexPicture;
    Bmp, NewBmp: TBitmap;
    x, y: integer;
begin
 // Rotate controls
 fpMain.Rotate(90, False);
 Bmp := Nil;
 NewBmp := Nil;
 try
  // Create temp bitmaps
  Bmp := TBitmap.Create;
  NewBmp := TBitmap.Create;
  // Process all selected controls
  for i:=0 to fpMain.SelectedCount-1 do begin
   // Skip all non-picture controls
   if not (fpMain.Selected[i] is TFlexPicture) then continue;
   Control := TFlexPicture(fpMain.Selected[i]);
   // Render source on Bmp
   if not Assigned(Control.PictureProp.Graphic) then continue;
   with Control.PictureProp.Graphic do begin
    Bmp.Width := Width;
    Bmp.Height := Height;
    Bmp.Canvas.Draw(0, 0, Control.PictureProp.Graphic);
   end;
   // Rotate Bmp pixels to NewBmp
   NewBmp.Width := Bmp.Height;
   NewBmp.Height := Bmp.Width;
   for y:=0 to Bmp.Height-1 do
    for x:=0 to Bmp.Width-1 do
     NewBmp.Canvas.Pixels[y, Bmp.Width-1-x] := Bmp.Canvas.Pixels[x, y];
   // Assign NewBmp to Graphic
   Control.PictureProp.Graphic.Assign(NewBmp);
  end;
 finally
  Bmp.Free;
  NewBmp.Free;
 end;
end;

procedure TfmMain.trbScaleChange(Sender: TObject);
begin
  fpMain.Zoom(trbScale.Position, Nil);
end;

procedure TfmMain.sbtScale100Click(Sender: TObject);
begin
  fpMain.Zoom(100, Nil);
end;

procedure TfmMain.tbtPickClick(Sender: TObject);
begin
 fpMain.CreatingControlClass := Nil;
 fpMain.ToolMode := ftmSelect;
end;

procedure TfmMain.tbtZoomClick(Sender: TObject);
begin
 fpMain.CreatingControlClass := Nil;
 fpMain.ToolMode := ftmZoom;
end;

procedure TfmMain.tbtPanClick(Sender: TObject);
begin
 fpMain.CreatingControlClass := Nil;
 fpMain.ToolMode := ftmPan;
end;

end.
