unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, FlexBase, FlexLibs, FlexUtils, FlexControls, FlexProps,
  ComCtrls;

type
  TfmMain = class(TForm)
    fpView: TFlexPanel;
    Splitter1: TSplitter;
    Panel1: TPanel;
    lbxLibItems: TListBox;
    Panel2: TPanel;
    btCreate: TButton;
    btCreateLib: TButton;
    sd_Lib: TSaveDialog;
    panDrag: TPanel;
    chClone: TCheckBox;
    Bevel1: TBevel;
    btSaveDoc: TButton;
    btLoadDoc: TButton;
    sd_Main: TSaveDialog;
    od_Main: TOpenDialog;
    sbrMain: TStatusBar;
    btAddToLib: TButton;
    procedure FormCreate(Sender: TObject);
    procedure btCreateClick(Sender: TObject);
    procedure btCreateLibClick(Sender: TObject);
    procedure btAddToLibClick(Sender: TObject);
    procedure btSaveDocClick(Sender: TObject);
    procedure btLoadDocClick(Sender: TObject);
    procedure lbxLibItemsStartDrag(Sender: TObject;
      var DragObject: TDragObject);
    procedure panDragStartDrag(Sender: TObject;
      var DragObject: TDragObject);
    procedure panDragMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure fpViewNotify(Sender: TObject; Control: TFlexControl;
      Notify: TFlexNotify);
  private
    { Private declarations }
    FMyLib: TFlexLibrary;
    FDragSource: TFlexPanel;
    procedure LoadLibrary(const Filename: string);
    function  CreateDragCloneObject(AControl: TFlexControl;
      DragName: string = ''): TFlexDragObject;
    procedure CheckTools;
    procedure RefreshLibListBox;
  public
    { Public declarations }
  end;

var
  fmMain: TfmMain;

implementation

{$R *.DFM}

procedure MyGetNameLinkByFlexPanel(FlexPanel: TFlexPanel;
  var NameLink: string);
begin
 NameLink := FlexPanel.Name;
 if NameLink = '' then
  // Check library
  if (FlexLibraries.Count > 0) and (FlexLibraries[0].Owner = FlexPanel) then
   NameLink := 'FlexLibraries';
end;

procedure MyGetFlexPanelByNameLink(NameLink: string;
  var FlexPanel: TFlexPanel);
// var Index: integer;
begin
 FlexPanel := Nil;
 if Assigned(fmMain) and Assigned(fmMain.fpView) and
    (NameLink = fmMain.fpView.Name) then
  FlexPanel := fmMain.fpView
 else
 // Check library
 if (NameLink = 'FlexLibraries') and (FlexLibraries.Count > 0) then
  FlexPanel := FlexLibraries[0].Owner {
 else begin
  // Check single libraries
  Index := SingleLibs.FindByName(NameLink);
  if Index >= 0 then FlexPanel := SingleLibs[Index].FlexPanel;
 end; }
end;

// TfmMain ////////////////////////////////////////////////////////////////////

procedure TfmMain.FormCreate(Sender: TObject);
begin
 LoadFlexCursors;
 fpView.InDesign := true;
 LoadLibrary(ExtractFilePath(ParamStr(0)) + 'test.fxl');
 CheckTools;
end;

procedure TfmMain.LoadLibrary(const Filename: string);
begin
 FMyLib.Free;
 lbxLibItems.Clear;
 // Create library
 FMyLib := FlexLibraries.New(Filename, TFlexLibraryWithIDs);
 // Fill list box
 RefreshLibListBox;
end;

procedure TfmMain.RefreshLibListBox;
var i: integer;
begin
 lbxLibItems.Items.BeginUpdate;
 try
  lbxLibItems.Clear;
  for i:=0 to FMyLib.Count-1 do
   lbxLibItems.Items.Add(FMyLib[i].Name);
  if lbxLibItems.Items.Count > 0 then lbxLibItems.ItemIndex := 0;
 finally
  lbxLibItems.Items.EndUpdate;
 end;
end;

procedure TfmMain.btCreateClick(Sender: TObject);
var Drag: TFlexDragObject;
    DragGroup: TFlexGroup;
begin
 if (lbxLibItems.Items.Count = 0) or (lbxLibItems.ItemIndex < 0) then exit;
 // Create drag group (for extract library item controls)
 Drag := FMyLib.Owner.CreateDragObject(FMyLib[lbxLibItems.ItemIndex], True, False);
 try
  DragGroup := Drag.DragGroup;
  Drag.DragGroup := Nil;
 finally
  Drag.Free;
 end;
 // Move DragGroup to fpView panel
 DragGroup.Owner := fpView;
 DragGroup.Parent := fpView.ActiveScheme;
 DragGroup.Layer := fpView.ActiveLayer;
 // Set DragGroup position
 DragGroup.Left := 20000;
 DragGroup.Top := 20000;
 // Ungroup
 fpView.UnselectAll;
 fpView.Select(DragGroup);
 fpView.Ungroup;
 // Test UserData
 with fpView.Selected[0] do begin
  UserData.Add('DoorCode');
  UserData.Add('DoorName');
 end;
end;

procedure TfmMain.btCreateLibClick(Sender: TObject);
var Lib: TFlexLibrary;
    LibItem: TFlexLibItem;
    Control: TFlexControl;
begin
 if sd_Lib.FileName = '' then
   sd_Lib.FileName := ExtractFilePath(ParamStr(0)) + 'MyLib.fxl';
 if not sd_Lib.Execute then exit;
 // Create new library
 Lib := FlexLibraries.New(sd_Lib.FileName);
 // Clear library
 Lib.Clear;
 // Add circle to libary
 Control := fpView.CreateControl(TFlexEllipse, 0, 0, 100000, 50000);
 fpView.UnselectAll;
 fpView.Select(Control);
 LibItem := Lib.NewFromFlex(fpView);
 LibItem.Name := Control.Name;
 Control.Free;
 // Add box to libary
 Control := fpView.CreateControl(TFlexBox, 0, 0, 100000, 50000);
 fpView.UnselectAll;
 fpView.Select(Control);
 LibItem := Lib.NewFromFlex(fpView);
 LibItem.Name := Control.Name;
 Control.Free;
 // Save library
 FlexLibraries.SaveLibrary(Lib);
 // Replace library in form
 LoadLibrary(Lib.LibFilename);
end;

procedure TfmMain.btAddToLibClick(Sender: TObject);
var ItemName: string;
    LibItem: TFlexLibItem;
begin
 if FlexLibraries.Count = 0 then exit;
 if not InputQuery('Insert new library item', 'Name:', ItemName) then exit;
 LibItem := FlexLibraries[0].NewFromFlex(fpView);
 LibItem.Name := ItemName;
 RefreshLibListBox;
end;

procedure TfmMain.lbxLibItemsStartDrag(Sender: TObject;
  var DragObject: TDragObject);
begin
 if (lbxLibItems.Items.Count = 0) or (lbxLibItems.ItemIndex < 0) then exit;
 // Create drag group
 if chClone.Checked then
  // Create clone from library item
  DragObject := CreateDragCloneObject(FMyLib[lbxLibItems.ItemIndex])
 else
  // Default drag object
  DragObject :=
    FMyLib.Owner.CreateDragObject(FMyLib[lbxLibItems.ItemIndex], True, False);
end;

procedure TfmMain.panDragStartDrag(Sender: TObject;
  var DragObject: TDragObject);
var Box: TFlexBox;
    MousePos: TPoint;
begin
 // Create hidden flex-panel for dragging controls
 if not Assigned(FDragSource) then FDragSource := TFlexPanel.Create(Self);
 // Create control for drag in source panel
 Box := TFlexBox(FDragSource.CreateControl(TFlexBox, 0, 0,
   panDrag.Width * PixelScaleFactor, panDrag.Height * PixelScaleFactor));
 Box.BrushProp.Color := panDrag.Color; // clLime;
 // Get mouse position
 GetCursorPos(MousePos);
 MousePos := (Sender as TControl).ScreenToClient(MousePos);
 // Create drag object
 DragObject :=
   Box.Owner.StartDrag(Box, False, False, MousePos,
     Point(0, 0), 100);
end;

procedure TfmMain.panDragMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
 if Button = mbRight then panDrag.Color := Random(256*256*256);
end;

function TfmMain.CreateDragCloneObject(AControl: TFlexControl;
  DragName: string = ''): TFlexDragObject;
var Clone: TFlexClone;
begin
 //Result := Nil;
 with AControl.Owner do begin
  History.DisableRecording;
  try
   // Create TFlexDragObject
   Result := TFlexDragObject.Create;
   with Result do
   try
    // Create DragGroup
    DragGroup := TFlexGroup.Create(AControl.Owner, Nil, Nil);
    DragGroup.ID := 0;
    DragGroup.IdProp.Style := DragGroup.IdProp.Style + [ psReadOnly ];
    // Create clone
    Clone := TFlexClone.Create(AControl.Owner, DragGroup, Nil);
    Clone.SourceProp.LinkedControl := AControl;
    // Initialize properties of DragGroup
    if Assigned(AControl) and (DragName = '') then DragName := AControl.Name;
    DragGroup.Name := DragName;
    DragGroup.Left := AControl.Left;
    DragGroup.Top := AControl.Top;
    DragGroup.Width := AControl.Width;
    DragGroup.Height := AControl.Height;
    DefaultFlex := AControl.Owner;
    MouseDragDelta := Point(0, 0);
   except
    Result.Free;
    raise;
   end;
  finally
   History.EnableRecording;
  end;
 end;
end;

procedure TfmMain.btSaveDocClick(Sender: TObject);
begin
 if not sd_Main.Execute then exit;
 fpView.SaveToFile(sd_Main.Filename);
 od_Main.Filename := sd_Main.Filename;
end;

procedure TfmMain.btLoadDocClick(Sender: TObject);
begin
 if not od_Main.Execute then exit;
 fpView.LoadFromFile(od_Main.Filename);
 sd_Main.Filename := od_Main.Filename;
end;

procedure TfmMain.fpViewNotify(Sender: TObject; Control: TFlexControl;
  Notify: TFlexNotify);
var Msg: string;
begin
 if (csDestroying in ComponentState) or (Notify <> fnSelect) then exit;
 Msg := '';
 if fpView.SelectedCount <> 1 then
  Msg := Format('Selected %d control(s)', [fpView.SelectedCount])
 else with fpView.Selected[0] do begin
  Msg := Format('%s: %s, ID=%d', [Name, ClassName, Id]);
  if fpView.Selected[0] is TFlexClone then
   Msg := Format('%s, Clone source: %s',
     [Msg, TFlexClone(fpView.Selected[0]).SourceProp.Value]);
 end;
 sbrMain.SimpleText := Msg;
 CheckTools;
end;

procedure TfmMain.CheckTools;
begin
 btAddToLib.Enabled := fpView.SelectedCount > 0;
end;

initialization
  GetNameLinkByFlexPanel := MyGetNameLinkByFlexPanel;
  GetFlexPanelByNameLink := MyGetFlexPanelByNameLink;

end.
 