unit FlexMultiPics;

interface

{$IFNDEF VER80}            { DELPHI 1.0     }
 {$IFNDEF VER90}           { DELPHI 2.0     }
  {$IFNDEF VER93}          { C++Builder 1.0 }
   {$IFNDEF VER100}        { DELPHI 3.0     }
    {$IFNDEF VER110}       { C++Builder 3.0 }
     {$IFNDEF VER120}      { DELPHI 4.0 }
      {$IFNDEF VER125}     { C++Builder 4.0 }
       {$IFNDEF VER130}    { DELPHI/C++Builder 5.0 }
        {$DEFINE D6_UP}
       {$ENDIF}
      {$ENDIF}
     {$ENDIF}
    {$ENDIF}
   {$ENDIF}
  {$ENDIF}
 {$ENDIF}
{$ENDIF}

uses
  {$IFDEF D6_UP} Variants, {$ENDIF}
  Windows, Graphics, Classes,
  FlexBase, FlexControls, FlexProps, FlexUtils;

type
  TFlexMultiPicControl = class(TFlexPicture)
  private
   FPictureIndexProp: TIntProp;
   FPictureListProp: TDataProp;
  protected
   FPictureList: TList;
   procedure ControlCreate; override;
   procedure ControlDestroy; override;
   procedure CreateProperties; override;
   procedure PropChanged(Sender: TObject; Prop: TCustomProp); override;
   procedure PropStored(Sender: TObject; Prop: TCustomProp;
     var IsStored: boolean; const PropName: string = ''); override;
   procedure PictureListItemChange(Sender: TObject);
   function  GetImage(Index: integer): TPicture;
   function  GetImageCount: integer;
  public
   procedure ClearPictureList;
   function  AddImage(ASource: TPicture = Nil): integer;
   procedure DeleteImage(Index: integer);
   procedure GetPictureListData(Sender: TObject; var Value: Variant);
   procedure SetPictureListData(Sender: TObject; var Value: Variant);
   property  PictureIndexProp: TIntProp read FPictureIndexProp;
   property  PictureListProp: TDataProp read FPictureListProp;
   property  ImageCount: integer read GetImageCount;
   property  Images[Index: integer]: TPicture read GetImage;
  end;

procedure VariantToPictureList(PicList: TList; var Value: Variant);
procedure PictureListToVariant(PicList: TList; var Value: Variant);

implementation

uses
  fMultiPic;

procedure VariantToPictureList(PicList: TList; var Value: Variant);
var ASize: integer;
    ACount: integer;
    i: integer;
    MS: TMemoryStream;
    ValueArray: pointer;
    Picture: TPicture;
    ReadStream: TStreamProc;
    WriteStream: TStreamProc;
begin
 if VarIsEmpty(Value) {or VarIsClear(Value) }or not VarIsArray(Value) then exit;
 ASize := VarArrayHighBound(Value, 1) +1;
 if ASize < 4 then exit;
 MS := TMemoryStream.Create;
 try
  ValueArray := VarArrayLock(Value);
  try
   MS.Write(ValueArray^, ASize);
  finally
   VarArrayUnlock(Value);
  end;
  MS.Position := 0;
  MS.Read(ACount, SizeOf(ACount));
  for i:=0 to ACount-1 do begin
   Picture := TPicture.Create;
   try
    FlexUtils.GetPicReadWrite(Picture, ReadStream, WriteStream);
    ReadStream(MS);
    PicList.Add(Picture);
   except
    Picture.Free;
    raise;
   end;
  end;
 finally
  MS.Free;
 end;
end;

procedure PictureListToVariant(PicList: TList; var Value: Variant);
var ACount: integer;
    i: integer;
    MS: TMemoryStream;
    ValueArray: pointer;
    ReadStream: TStreamProc;
    WriteStream: TStreamProc;
begin
 MS := TMemoryStream.Create;
 try
  ACount := PicList.Count;
  MS.Write(ACount, SizeOf(ACount));
  for i:=0 to ACount-1 do begin
   FlexUtils.GetPicReadWrite(TPicture(PicList[i]), ReadStream, WriteStream);
   WriteStream(MS);
  end;
  Value := VarArrayCreate([0, MS.Size-1], varByte);
  ValueArray := VarArrayLock(Value);
  try
   Move(MS.Memory^, ValueArray^, MS.Size);
  finally
   VarArrayUnlock(Value);
  end;
 finally
  MS.Free;
 end;
end;

// TFlexMultiPicControl ///////////////////////////////////////////////////////

procedure TFlexMultiPicControl.ControlCreate;
begin
 inherited;
 FPictureList := TList.Create;
end;

procedure TFlexMultiPicControl.ControlDestroy;
begin
 inherited;
 ClearPictureList;
 FPictureList.Free;
end;

procedure TFlexMultiPicControl.CreateProperties;
begin
 inherited;
 FPictureIndexProp := TIntProp.Create(Props, 'PictureIndex');
 FPictureListProp := TDataProp.Create(Props, 'PictureList');
 FPictureListProp.OnGetPropData := GetPictureListData;
 FPictureListProp.OnSetPropData := SetPictureListData;
 FPictureListProp.EditFormClass := TfmMultiPicEdit;
 FPictureListProp.Style := FPictureListProp.Style + [psVisible, psEditForm];
end;

procedure TFlexMultiPicControl.ClearPictureList;
var i: integer;
begin
 for i:=0 to FPictureList.Count-1 do TPicture(FPictureList[i]).Free;
 FPictureList.Clear;
end;

procedure TFlexMultiPicControl.PropChanged(Sender: TObject;
  Prop: TCustomProp);
begin
 if Assigned(FPictureList) and
   (FPictureIndexProp.Value >= 0) and
   (FPictureIndexProp.Value < FPictureList.Count) then begin
  if Prop = FPictureIndexProp then
   PictureProp.Graphic :=
     TPicture(FPictureList[FPictureIndexProp.Value]).Graphic
  else
  if Prop = FPictureListProp then
   // Direct picture list changing
   PictureListItemChange(FPictureList[FPictureIndexProp.Value]);
 end;
 inherited;
end;

procedure TFlexMultiPicControl.PropStored(Sender: TObject;
  Prop: TCustomProp; var IsStored: boolean; const PropName: string);
begin
 if (Prop = PictureProp) and (FPictureList.Count > 0) then
  IsStored := false
 else
 if Prop = FPictureIndexProp then
  IsStored := FPictureIndexProp.Value > 0
 else
 if Prop = FPictureListProp then
  IsStored := FPictureList.Count > 0
 else
  inherited;
end;

function TFlexMultiPicControl.AddImage(ASource: TPicture = Nil): integer;
var Picture: TPicture;
begin
 Picture := TPicture.Create;
 if Assigned(ASource) then Picture.Assign(ASource);
 Result := FPictureList.Add(Picture);
 Picture.OnChange := PictureListItemChange;
 if ((FPictureIndexProp.Value >= Result) and
     (FPictureIndexProp.Value < FPictureList.Count)) then
  PictureListItemChange(Images[FPictureIndexProp.Value]);
end;

procedure TFlexMultiPicControl.DeleteImage(Index: integer);
var Picture: TPicture;
begin
 Picture := TPicture(FPictureList[Index]);
 Picture.OnChange := Nil;
 FPictureList.Delete(Index);
 Picture.Free;
 if FPictureIndexProp.Value >= FPictureList.Count then
  FPictureIndexProp.Value := FPictureList.Count - 1
 else
 if FPictureIndexProp.Value >= Index then
  PictureListItemChange(Images[FPictureIndexProp.Value]);
end;

function TFlexMultiPicControl.GetImage(Index: integer): TPicture;
begin
 Result := TPicture(FPictureList[Index]);
end;

function TFlexMultiPicControl.GetImageCount: integer;
begin
 Result := FPictureList.Count;
end;

procedure TFlexMultiPicControl.PictureListItemChange(Sender: TObject);
var Index: integer;
begin
 Index := FPictureList.IndexOf(Sender);
 if Index = FPictureIndexProp.Value then 
  // Reload current picture
  PictureProp.Graphic := TPicture(Sender).Graphic;
end;

procedure TFlexMultiPicControl.GetPictureListData(Sender: TObject;
  var Value: Variant);
begin
 PictureListToVariant(FPictureList, Value);
end;

procedure TFlexMultiPicControl.SetPictureListData(Sender: TObject;
  var Value: Variant);
var i: integer;
begin
 ClearPictureList;
 VariantToPictureList(FPictureList, Value);
 for i:=0 to FPictureList.Count-1 do
  TPicture(FPictureList[i]).OnChange := PictureListItemChange;
 // Refresh current image
 if (FPictureIndexProp.Value >= 0) and
    (FPictureIndexProp.Value < FPictureList.Count) then
  PictureListItemChange(FPictureList[FPictureIndexProp.Value]);
end;

///////////////////////////////////////////////////////////////////////////////

procedure RegisterFlexMultiPicControl;
begin
 RegisterFlexControl(TFlexMultiPicControl);
end;

initialization
  RegisterFlexMultiPicControl;

end.
