unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtDlgs, ComCtrls, StdCtrls, Buttons, ExtCtrls,
  FlexBase, FlexUtils, FlexMultiPics;

type
  TfmMain = class(TForm)
    fpView: TFlexPanel;
    Panel1: TPanel;
    Bevel1: TBevel;
    bbAdd: TBitBtn;
    bbDelete: TBitBtn;
    cbImages: TComboBox;
    bbSave: TBitBtn;
    bbLoad: TBitBtn;
    bbEdit: TBitBtn;
    StatusBar1: TStatusBar;
    opd_Picture: TOpenPictureDialog;
    od_Flex: TOpenDialog;
    sd_Flex: TSaveDialog;
    procedure FormCreate(Sender: TObject);
    procedure bbAddClick(Sender: TObject);
    procedure bbDeleteClick(Sender: TObject);
    procedure cbImagesChange(Sender: TObject);
    procedure bbEditClick(Sender: TObject);
    procedure bbSaveClick(Sender: TObject);
    procedure bbLoadClick(Sender: TObject);
  private
    FMultiPic: TFlexMultiPicControl;  
    procedure CheckTools;
    procedure CreateMultiPic;
    procedure RebuildImageCombo;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  fmMain: TfmMain;

implementation

{$R *.DFM}

procedure TfmMain.FormCreate(Sender: TObject);
begin
 LoadFlexCursors;
 fpView.InDesign := TRUE;
 CreateMultiPic;
 RebuildImageCombo;
 CheckTools;
end;

procedure TfmMain.CreateMultiPic;
begin
 FMultiPic := TFlexMultiPicControl.Create(fpView, fpView.ActiveScheme,
   fpView.ActiveLayer);
 FMultiPic.Left := 10000;
 FMultiPic.Top := 10000;
 FMultiPic.Width := 100000;
 FMultiPic.Height := 100000;
 FMultiPic.IsSelected := true;
end;

procedure TfmMain.CheckTools;
begin
 //bbEdit.Enabled := cbImages.Items.Count > 0;
 bbDelete.Enabled := cbImages.Items.Count > 0;
 cbImages.Enabled := cbImages.Items.Count > 0;
end;

procedure TfmMain.RebuildImageCombo;
begin
 cbImages.Items.BeginUpdate;
 try
  // Add new images
  while cbImages.Items.Count < FMultiPic.ImageCount do
   cbImages.Items.Add(IntToStr(cbImages.Items.Count));
  // Delete extra images
  while cbImages.Items.Count > FMultiPic.ImageCount do
   cbImages.Items.Delete(cbImages.Items.Count - 1);
  // Set current
  cbImages.ItemIndex := FMultiPic.PictureIndexProp.Value;
 finally
  cbImages.Items.EndUpdate;
 end;
end;

procedure TfmMain.bbAddClick(Sender: TObject);
var Picture: TPicture;
    Index: integer; 
begin
 if not opd_Picture.Execute then exit;
 Picture := TPicture.Create;
 try
  Picture.LoadFromFile(opd_Picture.FileName);
  Index := FMultiPic.AddImage(Picture);
  FMultiPic.PictureIndexProp.Value := Index;
 finally
  Picture.Free;
 end;
 RebuildImageCombo;
 CheckTools;
end;

procedure TfmMain.bbDeleteClick(Sender: TObject);
begin
 FMultiPic.DeleteImage(FMultiPic.PictureIndexProp.Value);
 RebuildImageCombo;
 CheckTools;
end;

procedure TfmMain.cbImagesChange(Sender: TObject);
begin
 FMultiPic.PictureIndexProp.Value := cbImages.ItemIndex;
end;

procedure TfmMain.bbEditClick(Sender: TObject);
begin
 FMultiPic.PictureListProp.Edit;
 RebuildImageCombo;
 CheckTools;
end;

procedure TfmMain.bbSaveClick(Sender: TObject);
begin
 if sd_Flex.Execute then fpView.SaveToFile(sd_Flex.FileName);
end;

procedure TfmMain.bbLoadClick(Sender: TObject);
var i: integer;
begin
 if not od_Flex.Execute then exit;
 FMultiPic := Nil;
 fpView.LoadFromFile(od_Flex.FileName);
 // Find first multi picture
 for i:=0 to fpView.ActiveScheme.Count-1 do
  if fpView.ActiveScheme[i] is TFlexMultiPicControl then begin
   FMultiPic := TFlexMultiPicControl(fpView.ActiveScheme[i]);
   FMultiPic.IsSelected := true;
   break;
  end;
 if not Assigned(FMultiPic) then begin
  ShowMessage('TFlexMultiPicControl object not found. Create new');
  CreateMultiPic;
 end;
 // Refresh
 sd_Flex.FileName := od_Flex.FileName;
 RebuildImageCombo;
 CheckTools;
end;

end.
