unit fMultiPic;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtDlgs, ImgList, StdCtrls, Buttons, ComCtrls,
  FlexBase, FlexProps, FlexControls, FlexMultiPics;

type
  TfmMultiPicEdit = class(TForm)
    Label1: TLabel;
    lvPictures: TListView;
    bbClose: TBitBtn;
    bbAdd: TBitBtn;
    bbDelete: TBitBtn;
    bbCancel: TBitBtn;
    imgIcons: TImageList;
    opd_Picture: TOpenPictureDialog;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure bbAddClick(Sender: TObject);
    procedure bbDeleteClick(Sender: TObject);
    procedure lvPicturesDblClick(Sender: TObject);
  private
    { Private declarations }
    FDataProp: TDataProp;
    FIndexProp: TIntProp;
    FPictureList: TList;
    procedure CheckTools;
    procedure LoadFromData;
    procedure SaveToData;
    procedure BuildImageList;
    procedure ClearPictureList;
    function  GetPictureIndex: integer;
    procedure SetPictureIndex(Value: integer);
  protected
    property  PictureIndex: integer read GetPictureIndex write SetPictureIndex;
  public
    { Public declarations }
  end;

var
  fmMultiPicEdit: TfmMultiPicEdit;

implementation

{$R *.DFM}

procedure TfmMultiPicEdit.FormCreate(Sender: TObject);
begin
 FPictureList := TList.Create;
end;

procedure TfmMultiPicEdit.FormDestroy(Sender: TObject);
begin
 if ModalResult = mrOk then SaveToData;
 ClearPictureList;
 FPictureList.Free;
end;

procedure TfmMultiPicEdit.FormShow(Sender: TObject);
var Control: TObject;
begin
 if (Tag <> 0) and (TObject(Tag) is TDataProp) then begin
  FDataProp := TDataProp(Tag);
  // Try to find picture index property
  Control := FDataProp.Owner.Owner;
  if Control is TFlexControl then
   FIndexProp := TIntProp(TFlexControl(Control).Props['PictureIndex']);
 end;
 LoadFromData;
 BuildImageList;
 // Set current image
 if Assigned(FIndexProp) and (FIndexProp.Value < FPictureList.Count) then
  PictureIndex := FIndexProp.Value;
end;

function TfmMultiPicEdit.GetPictureIndex: integer;
begin
 Result := lvPictures.Items.IndexOf(lvPictures.ItemFocused);
end;

procedure TfmMultiPicEdit.SetPictureIndex(Value: integer);
begin
 lvPictures.ItemFocused := lvPictures.Items.Item[Value];
end;

type
  TDataPropFriend = class(TDataProp);

procedure TfmMultiPicEdit.LoadFromData;
var Data: Variant;
begin
 if Assigned(FDataProp) then begin
  Data := TDataPropFriend(FDataProp).GetPropValue('');
  VariantToPictureList(FPictureList, Data);
 end;
end;

procedure TfmMultiPicEdit.SaveToData;
var Data: Variant;
    Control: TObject;
begin
 if Assigned(FDataProp) then begin
  PictureListToVariant(FPictureList, Data);
  Control := FDataProp.Owner.Owner;
  if Control is TFlexMultiPicControl then
   TFlexMultiPicControl(Control).SetPictureListData(FDataProp, Data)
  else
   TDataPropFriend(FDataProp).SetPropValue('', Data);
 end;
 if Assigned(FIndexProp) then FIndexProp.Value := PictureIndex;
end;

procedure TfmMultiPicEdit.BuildImageList;
var Bmp: TBitmap;
    i: integer;
    Item: TListItem; 
begin
 Bmp := Nil;
 lvPictures.Items.BeginUpdate;
 try
  lvPictures.Items.Clear;
  imgIcons.Clear;
  Bmp := TBitmap.Create;
  Bmp.Width := imgIcons.Width;
  Bmp.Height := imgIcons.Height;
  // Load images to list view
  for i:=0 to FPictureList.Count-1 do begin
   Bmp.Canvas.Brush.Color := clWindow;
   Bmp.Canvas.FillRect(Rect(0, 0, Bmp.Width, Bmp.Height));
   Bmp.Canvas.StretchDraw(Rect(0, 0, Bmp.Width, Bmp.Height),
     TPicture(FPictureList[i]).Graphic);
   imgIcons.Add(Bmp, Nil);
   Item := lvPictures.Items.Add;
   Item.Caption := IntToStr(i);
   Item.ImageIndex := i;
  end;
 finally
  lvPictures.Items.EndUpdate;
  Bmp.Free;
 end;
end;

procedure TfmMultiPicEdit.CheckTools;
begin
 bbDelete.Enabled := (PictureIndex >= 0);
end;

procedure TfmMultiPicEdit.ClearPictureList;
var i: integer;
begin
 for i:=0 to FPictureList.Count-1 do TPicture(FPictureList[i]).Free;
 FPictureList.Clear;
end;

procedure TfmMultiPicEdit.bbAddClick(Sender: TObject);
var NewIndex: integer;
    Picture: TPicture;
begin
 if not opd_Picture.Execute then exit;
 Picture := TPicture.Create;
 try
  Picture.LoadFromFile(opd_Picture.FileName);
  NewIndex := FPictureList.Add(Picture);
 except
  Picture.Free;
  raise;
 end;
 BuildImageList;
 PictureIndex := NewIndex;
 CheckTools;
end;

procedure TfmMultiPicEdit.bbDeleteClick(Sender: TObject);
var Index: integer;
    Picture: TPicture;
begin
 Index := PictureIndex;
 if Index < 0 then exit;
 Picture := TPicture(FPictureList[Index]);
 FPictureList.Delete(Index);
 Picture.Free;
 BuildImageList;
 if Index < lvPictures.Items.Count
  then PictureIndex := Index
  else PictureIndex := lvPictures.Items.Count -1;
 CheckTools;
end;

procedure TfmMultiPicEdit.lvPicturesDblClick(Sender: TObject);
begin
 ModalResult := mrOk;
end;

end.
